.define TI8X

; ===========================================================
; = ASSUME THIS WILL BRICK YOUR CALC
; = IT RELIES ON UNLOCKED FLASH
; ===========================================================

.define VERSION 1

.ifdef TI83
 #include "ti83.inc"
 .binarymode TI83
 .variablename "FLASHFUN"
 .define scratchMem plotSScreen
 .unsquish
 .org asm_exec_ram
.elseifdef TI8X
 #include "ti83plus.inc"
 .binarymode TI8X
 .variablename "FLASHFUN"
 .define scratchMem appBackupScreen
 .org userMem-2
 .db t2ByteTok,tAsmCmp
.elseifdef TI84PCSE
 #include "ti84plus.inc"
 .binarymode TI8X
 .variablename "FLASHFUN"
 .define scratchMem appBackupScreen
 .org userMem-2
 .db tExtTok,tAsm84CCmp
.endif

;First:
; di
; push ix
; push iy
; call FlashUnlock
; pop iy
; pop ix
; in a,($2)
; ld hl,MsgFlashUnlocked
; and %00000100
; jr nz,FlashUnlocked2
; ret
; FlashUnlocked2:
; bcall(_puts)
; call flashlock
; ret

Main:
 di
 ;push iy
 ;call FlashUnlock
 ;pop iy
 call UnlockFlash
 ;jr c,UnlockFlashFailed
 in a,($2)
 ;ld hl,MsgFlashUnlocked
 and %00000100
 jr nz,FlashUnlocked
 ;ld hl,MsgFlashLocked
 ;bcall(_puts)
 ;bcall(_newline)
 ret
 FlashUnlocked:
 ;bcall(_puts)
 ;bcall(_NewLine)
 ;call WaitForKey
 di
 ld bc,VERSION
 call PutBCinResults
 call EnterAutoSelect
 call GetManufacturerID
 call GetDeviceID
 ; attempt CFI mode. If we don't have CFI, just dump results
 call EnterCFI
 ; if we're in CFI, reading from $20 gives $51
 ld a,($0020)
 cp $51
 jr nz,PrepValueDumping
 ; CFI manufacturer ID is in $0026
 ld hl,$0026
 call Read16
 ; CFI has a pointer to the start of the primary vendor specific table at $002A
 ld a,($002A)
 add a,a
 ld b,0
 ld c,a
 call Get16
 ld hl,6
 add hl,bc
 call Read16 ; read mversion number (seems to be broken)
 ; when exiting CFI, reset command takes you back to autoselect
 call ResetFlash
 ; from here we should check for locked sectors. put FFFF on OP1,
 ;  (ld bc,$0FFFF \ bcall(_SetXXXXOP2) \ bcall(_OP2ToOP1) \ 
 ;   bcall(_Plus1) \ add OP2 to the list) as a divider
 ; then just start adding locked sectors to the end of the list
 PrepValueDumping:
 call ResetFlash ; we can start using ROM again
 call ShamefulExit ; we can safely use system calls again
 jp ListSetup
 ;jp ShamefulExit
; check if we are on a non-83 ASIC

Read16FromA:
 ; address in a
 ; returned data in bc
 ; af destroyed
 ld h,0
 ld a,l
Read16:
 ; address in hl
 ; returned data in bc
 ; af destroyed
 ld a,(hl)
 inc hl
 inc hl
 ld c,a
 ld a,(hl)
 ld b,a
 jp PutBCInResults

Read8:
 ld a,(hl)
 ld b,0
 ld c,a
 jp PutBCInResults

Get16:
 ld a,(hl)
 inc hl
 inc hl
 ld c,a
 ld a,(hl)
 ld b,a
 ret

ShamefulExit:
 call ResetFlash
 call flashlock
 ei
 ret
 
 
EnterAutoSelect:
 ; flash must be unlocked and interrupts disabled
 ld a,$AA
 ld ($0AAA),a
 ld a,$55
 ld ($0555),a
 ld a,$90
 ld ($0AAA),a
 ret

GetManufacturerID:
 ld b,0
 ld a,($0000)
 ld c,a
 ld a,($0200)
 cp c
 jr z,GetManufacturerID_8Bit
 ld b,a
 GetManufacturerID_8Bit:
 jp PutBCInResults

GetDeviceID:
 ld b,0
 ld a,($0002)
 ld c,a
 ld a,($0202)
 cp c
 jr z,GetDeviceID_8Bit
 ld b,a
 GetDeviceID_8Bit:
 jp PutBCInResults

EnterCFI:
 ; flash must be unlocked and interrupts disabled
 ld a,$98
 ld ($0AAA),a
 ret
 
 
ResetFlash:
 ; flash must be unlocked and interrupts disabled
 ld a,$F0
 ld ($0000),a
 ret
 
 
; code grabbed from undoc.asm but hastily made 16 bit
 
PutResultInResults:
 ld a,(Result)
 ld c,a
 ld a,(Result+1)
 ld b,a
 jr PutBCInResults
PutAInResults:
 ld b,0
 ld c,a
PutBCInResults:
 push bc
  push hl
   ld hl,scratchMem
   ld de,(ResultsPtr)
   add hl,de
   add hl,de
   ld a,c
   ld (hl),a
   inc hl
   ld a,b
   ld (hl),a
   inc de
   ld (ResultsPtr),de
   ld a,0
   ld (Result),a
   ld (Result+1),a
  pop hl
 pop bc
 ret


ListSetup:
 ; entry here: create L1 (delete if exists), put values from
 ;             scratchMem into L1. Num. entries
 ld hl,L1name
 .ifdef Flash
  rst rMov9ToOP1
  rst rFindSym
 .elifdef TI83
  bcall(_Mov9ToOP1)
  rst rFindSym
 .else
  bcall(_Mov9ToOP1)
  bcall(_FindSym)
 .endif
 jr c,ListDontDelete
 .ifndef TI82
  bcall(_DelVarArc)
 .endif
ListDontDelete:
 ld hl,(ResultsPtr)
 bcall(_createrlist)
 inc de
 inc de ; skip length info
 ld hl,(ResultsPtr)
 ld b,l ; let's not run more than 255 tests
 ld a,b
 cp 0
 jr z,ListEmpty
 ld hl,scratchMem ; start of list in hl
 ListSetValues:
  push bc ; preserve loop counter in b
   push hl ; preserve results pointer
    push de ; preserve list element pointer, we use this first
     push hl ; is this part necessary? maybe for movfrop1?
      bcall(_ldHLind)
      bcall(_SetXXXXOP2)
      bcall(_OP2ToOP1)
     pop hl
    pop de
    bcall(_MovFrOP1) ; put OP1 in list, inc de by 9
   pop hl
   inc hl ; next word in results
   inc hl
  pop bc
  djnz ListSetValues
ListExit:
 ; if we didn't use appBackupScreen, mark the graph memory as dirty
 .ifdef TI83
  set graphdraw,(iy+graphflags)
 .endif
 ret
ListEmpty: ; should never be called ; WILL KILL VAT ON 86 RIGHT NOW
 ld a,0
 push de
 .ifndef TI82
  bcall(_setxxop1)
 .else
  push hl
  ld hl,0
  ; try to find romcall for 0 in op1
  bcall(_SetXXXXOP2)
  bcall(_OP2ToOP1)
  pop hl
 .endif
 pop de
 ListEmptyLoop:
  push bc
  bcall(_MovFrOP1)
  pop bc
  djnz ListEmptyLoop
 jr ListExit


Result:
 .dw 0
ResultsPtr:
 .dw 0
 
L1name:
 .ifdef TI73
  .db 0 ; format????????????
 .elseifdef TI83
  .db ListObj,tVarLst,tL1,0,0
 .elseifdef TI8X
  .db ListObj,tVarLst,tL1,0,0
 .endif

;WaitForKey:
; ; since we don't have any ROM, we need to do it from scratch
; di
; push af
; ld a,$0FF
; out ($01),a
; ld a,$0FD
; out ($01),a
; ; necessary delay
; ex (sp),hl
; ex (sp),hl
; WaitForKey_Loop:
;  in a,($01)
;  and %01000000
;  jr nz,WaitForKey_Loop
; ei
; ret
 
flashlock:
 .ifdef TI8X
 bcall(_FlashWriteDisable)
 .endif
 ret
 
; ------------------------------------------------------------------
; --- not my code
; ------------------------------------------------------------------
;flashunlock
;
;	ld a,$14
;	ld bc,flashunlock_ram_end-flashunlock_ram_start
;	ld de,flashunlock_ram_start-flashunlock_return+$81E3
;	ld hl,$8167
;	ld iy,$0031 ; must be $0031
;	ld ($83EE),a ; must be $08>$15
;	ld ($84DB),hl ; must be $8167
;	ld ($9834),a ; must be $03>$FF
;	add a,l
;	ld ($983A),a ; must be close enough to but under $80
;	ld hl,flashunlock_ram_start
;	ldir
;
;	in a,($06)
;	push af
;
;	in a,($02)
;	rra
;	or %10111111
;	ld d,a
;	and $7B
;
;	jp flashunlock_ram_start-flashunlock_return+$81E3
;
;flashunlock_ram_start
;
;	out ($06),a
;
;	ld hl,($5092)
;
;	ld a,d
;	and $7C
;	out ($06),a
;
;	ld a,$10
;	cpir
;
;	jp (hl)
;
;flashunlock_return
;
;	ld hl,24
;	add hl,sp
;	ld sp,hl
;	ld hl,$0007
;	ld (hl),$FF
;	ld b,%01000000
;
;	pop af
;	out ($06),a
;
;flashunlock_wait
;
;	ld a,(hl)
;
;	rla
;	ret c
;
;	and b
;	jp z,flashunlock_wait-flashunlock_return+$81E3
;
;	ld a,(hl)
;
;	rla
;	ret c
;
;	ld (hl),$F0
;
;	ret
;
;flashunlock_ram_end
;
;
;
;
;flashlock
;
;	in a,($06)
;	ld b,a
;
;	in a,($02)
;	rra
;	or %10111111
;	ld c,a
;	and $7B
;	out ($06),a
;
;	ld hl,($4F3C)
;
;	ld a,c
;	and $7C
;	out ($06),a
;
;	call flashlock_jump
;
;	ld a,b
;	out ($06),a
;
;	ret
;
;flashlock_jump
;
;	jp (hl)


.ifdef TI73
jumpBackup      equ     appBackUpScreen     ;2
unlockLocation  equ     jumpBackup+2        ;?
jumpTable       equ     unlockLocation+512  ;50
spBackup        equ     jumpTable+50        ;2
oldStack        equ     spBackup+2          ;50

UnlockFlash:
;BrandonW's flash unlock routine from Chameleon
;Unlocks Flash protection.
;Destroys appBackUpScreen, iMathPtr4, iMathPtr5,
;  and probably other stuff.
        ld ix,searchPattern
        ld de,4000h
        ld a,1Ch
        call FindPattern
        jr nc,patternFound
        ld ix,searchPattern
        ld de,4000h
        ld a,1Dh
        call FindPattern
        ret c
patternFound:
        ld ix,(appBackUpScreen)
        ld hl,unlockStart
        ld de,unlockLocation
        ld bc,unlockEnd-unlockStart
        ldir
        jp unlockLocation
unlockStart:
        ld hl,(8080h)
        push hl
        ld hl,(8082h)
        push hl
        ld hl,8080h
        ld (hl),0C3h
        inc hl
        ld de,returnPoint-unlockStart+unlockLocation
        ld (hl),e
        inc hl
        ld (hl),d
        in a,(6)
        push af
        ld hl,jumpTable
        ld (hl),80h
        ld de,jumpTable+1
        ld bc,49
        ldir
        ld (spBackup),sp
        ld hl,(spBackup)
        ld de,oldStack
        ld bc,50
        ldir
        ld a,1Ch
        out (6),a
        ld hl,(spBackup)
        ld de,-8
        add hl,de
        ld de,jumpTable
        ex de,hl
        ld bc,50
        jp (ix)
returnPoint:
        ld sp,(spBackup)
        ld de,(spBackup)
        ld hl,oldStack
        ld bc,50
        ldir
        pop af
        out (6),a
        pop hl
        ld (8082h),hl
        pop hl
        ld (8080h),hl
        xor a
        ret
unlockEnd:        

FindPattern:
;Pattern in IX, Flash page in A, starting address in DE
;Returns NZ if pattern not found
;(appBackUpScreen) contains the address of match found
;Search pattern: terminated by 0FEh
;                0FFh is ? (one-character wildcard)
        ld (iMathPtr4),a
        in a,(6)
        push af
        ld hl,findPatternRet
        push hl
        ld a,(iMathPtr4)
        out (6),a
        dec de
searchLoopRestart:
        inc de
        ld (appBackUpScreen),de
        push ix
        pop hl
searchLoop:
        ld b,(hl)
        ld a,0FEh
        cp b
        ld a,b
        ret z
        inc de
        inc a
        jr z,matchSoFar
        dec de
        ld a,(de)
        inc de
        bit 7,d
        ret nz
        cp b
        jr z,matchSoFar
        ld de,(appBackUpScreen)
        jr searchLoopRestart
matchSoFar:
        inc hl
        jr searchLoop
findPatternRet:
        pop bc
        ld a,b
        out (6),a
        ret
searchPattern:
        .db 0F5h,3Eh,01h,00h,0F3h,00h,00h
        .db 0EDh,56h,0F3h,0D3h,14h,0F3h,0CDh,0FFh
        .db 0FFh,0F1h,0EFh,54h,80h,0FEh
.else
UnlockFlash:
;Unlocks Flash protection.
;Destroys: appBackUpScreen
;          pagedCount
;          pagedGetPtr
;          arcInfo
;          iMathPtr5
;          pagedBuf
;          ramCode
        in a,(6)
        push af
        ld a,7Bh
        call translatePage
        out (6),a
        ld hl,5092h
        ld e,(hl)
        inc hl
        ld d,(hl)
        inc hl
        ld a,(hl)
        call translatePage
        out (6),a
        ex de,hl
        ld a,0CCh
        ld bc,0FFFFh
        cpir
        ld e,(hl)
        inc hl
        ld d,(hl)
        push de
        pop ix
        ld hl,9898h
        ld (hl),0C3h
        inc hl
        ld (hl),returnPoint & 11111111b
        inc hl
        ld (hl),returnPoint >> 8
        ld hl,pagedBuf
        ld (hl),98h
        ld de,pagedBuf+1
        ld bc,49
        ldir
        ld (iMathPtr5),sp
        ld hl,(iMathPtr5)
        ld de,9A00h
        ld bc,50
        ldir   
        ld de,(iMathPtr5)
        ld hl,-16
        add hl,de
        ld (iMathPtr5),hl
        ld iy,0056h-25h
        ld a,50
        ld (pagedCount),a
        ld a,8
        ld (arcInfo),a
        jp (ix)
translatePage:
        ld b,a
        in a,(2)
        and 80h
        jr z,_is83P
        in a,(21h)
        and 3
        ld a,b
        ret nz
        and 3Fh
        ret
_is83P: ld a,b
        and 1Fh
        ret
returnPoint:
        ld iy,flags
        ld hl,(iMathPtr5)
        ld de,16
        add hl,de
        ld sp,hl
        ex de,hl
        ld hl,9A00h
        ld bc,50
        ldir
        pop af
        out (6),a
        ret
.endif
 
 
; MsgFlashLocked:
;  .db "Flash Locked",0
; MsgFlashUnlocked:
;  .db "Flash Unlocked",0
 
.ifdef TI83
 ; TI-83 nostub has a squished footer
 .squish
 .db tenter,tend,tenter,t0,t0,t0,t0,tenter,tend
.endif